/* 
 * Copyright 1993, 1994 Liverpool University Computer Science Department
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided 
 * that the above copyright notice appear in all copies and that both that 
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the name of L.U.C.S. not be used in advertising
 * or publicity pertaining to distribution of the software without specific, 
 * written prior permission. L.U.C.S. makes no representations about the 
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * FILE NAME:	XbrText.c
 * CREATED:	Wed Oct 27 1993
 * AUTHOR:	Rik Turnbull
 * DESCRIPTION: Contains routines for manipulating the XmText widget.
 *
 */

#include <Xm/Xm.h>	/* Needed for Motif code                         */
#include <Xm/Text.h>    /* XmTextGetString() etc.                        */

#include <stdio.h>	/* Lots - BUFSIZ, FILE etc.                      */
#include <stdarg.h>	/* For variable argument handling routines       */

#include "XbrText.h"	/* XbrWARNING, XbrMESSAGE                        */

void XbrTextAppend(Widget, int, ...);		/* Append text to widget */
void XbrTextSet(Widget, int, ...);		/* Set text in widget    */
void XbrTextFSet(Widget, int, ...);		/* Text in XmTextField   */
int XbrTextHighlight(Widget, char *);		/* Highlight string      */
int XbrTextDump(Widget, char *);		/* Dump widget to a file */

/* XbrTextAppend:*********************************************************/
/* XbrTextAppend: Routine to append text to a given text widget. There   */
/* XbrTextAppend: are currently 2 types of messages:                     */
/* XbrTextAppend:     XbrMESSAGE    -    Display the text                */
/* XbrTextAppend:     XbrWARNING    -    Display the text and sound the  */
/* XbrTextAppend:                        bell                            */
/* XbrTextAppend: The text takes the form of printf style arguments.     */
/* XbrTextAppend:*********************************************************/

void XbrTextAppend(Widget w, int msg_type, ...)
{
    char *fmt, buffer[BUFSIZ];
    XmTextPosition pos;
    va_list argp;

    /* If it is a warning beep! */
    if(msg_type == XbrWARNING)
        XBell(XtDisplay(w), 50);

    /* Get format string */
    va_start(argp, msg_type);
    fmt = va_arg( argp, char *);
    vsprintf(buffer, fmt, argp);
    va_end(argp);

    /* Find the insertion position */
    pos = XmTextGetLastPosition(w);
    XmTextReplace(w, pos, pos, buffer);
}

/* XbrTextFSet:************************************************************/
/* XbrTextFSet: Routine to set the text in a given text widget. There     */
/* XbrTextFSet: are currently 2 types of messages:                        */
/* XbrTextFSet:     XbrMESSAGE    -    Display the text                   */
/* XbrTextFSet:     XbrWARNING    -    Display the text and sound the     */
/* XbrTextFSet:                        bell                               */
/* XbrTextFSet: The text takes the form of printf style arguments.        */
/* XbrTextFSet:************************************************************/

void XbrTextFSet(Widget w, int msg_type, ...)
{
    char *fmt, buffer[BUFSIZ];
    va_list argp;

    /* If it is a warning beep! */
    if(msg_type == XbrWARNING)
        XBell(XtDisplay(w), 50);

    /* Get format string */
    va_start(argp, msg_type);
    fmt = va_arg( argp, char *);
    vsprintf(buffer, fmt, argp);
    va_end(argp);

    /* Set the text string and update the display */
    XmTextFieldSetString(w, buffer);
    XmUpdateDisplay(w);
}

/* XbrTextSet:************************************************************/
/* XbrTextSet: Routine to set the text in a given text widget. There     */
/* XbrTextSet: are currently 2 types of messages:                        */
/* XbrTextSet:     XbrMESSAGE    -    Display the text                   */
/* XbrTextSet:     XbrWARNING    -    Display the text and sound the     */
/* XbrTextSet:                        bell                               */
/* XbrTextSet: The text takes the form of printf style arguments.        */
/* XbrTextSet:************************************************************/

void XbrTextSet(Widget w, int msg_type, ...)
{
    char *fmt, buffer[BUFSIZ];
    va_list argp;

    /* If it is a warning beep! */
    if(msg_type == XbrWARNING)
        XBell(XtDisplay(w), 50);

    /* Get format string */
    va_start(argp, msg_type);
    fmt = va_arg( argp, char *);
    vsprintf(buffer, fmt, argp);
    va_end(argp);

    /* Set the text string and update the display */
    XmTextSetString(w, buffer);
    XmUpdateDisplay(w);
}

/* XbrTextHighlight:******************************************************/
/* XbrTextHighlight: Highlight the given search term in the text widget. */ 
/* XbrTextHighlight:******************************************************/

int XbrTextHighlight(Widget w, char *search_term)
{
    char *contents;
    int len, indx = 0;
    XmTextPosition start, end, pos;

    /* Get the contents of the text widget */
    contents = XmTextGetString(w);

    /* Get the length of the search term */
    len = strlen(search_term);

    /* Search for the term in the buffer - it's pretty confusing but it
       seems to do the trick :-)
    */
    while(pos = search_matches(search_term, &contents[indx], 0)) {
        start = indx + pos - 1;
        end = len + indx + pos - 1;
        XmTextSetHighlight(w, start, end, XmHIGHLIGHT_SELECTED);
        indx += (pos+1);
        /* Should not happen */
        if(indx > len)
            return(0);
    }

    return(1);
}

/* XbrTextDump:**********************************************************/
/* XbrTextDump: Dump all the text in a widget to a file.                */
/* XbrTextDump:**********************************************************/

int XbrTextDump(Widget w, char *filename)
{
    FILE *fp;

    /* Open the file */
    if((fp = fopen(filename, "w")) == NULL)
        return(0);

    /* Dump the text */
    fprintf(fp, "%s", XmTextGetString(w));

    /* Close the file */
    fclose(fp);

    /* Easy eh? */
    return(1);
}
