/*
 * Copyright 1993 University of Liverpool Computer Science Department
 *
 * Unauthorised duplication or distribution of all or part of this code is
 * not permitted. This software is to be used only for accessing the HP-UX
 * specific public domain archives  at the University of Liverpool and its
 * official mirror sites and the commercial SoftNet archive. So there.
 *
 * FILE NAME:	tmpfile.c
 * CREATED:	Thu Nov 11 1993
 * AUTHOR:	Rik Turnbull (rik@csc.liv.ac.uk)
 * DESCRIPTION:	Temporary file handling routines
 *
 */

#include <stdio.h>

/* Temporary dir */
#define	TMPDIR	"/tmp"

/* Data structure describing a temporary file */
typedef struct tmp_struct {
    char *name;
    struct tmp_struct *next;
} TMP_FILE;

/* Private functions */
static TMP_FILE *tmpfile_alloc(char *);

/* Functions for the outside world */
void tmpfile_free();
int  tmpfile_add(char *);
int  tmpfile_search(char *);
char *tmpfile_create(char *, char *);

/* First file in linked list */
static TMP_FILE *first_tmpfile = NULL;

/* tmpfile_alloc:*************************************************************/
/* tmpfile_alloc: Allocate memory for a new TMP_FILE.                        */
/* tmpfile_alloc:*************************************************************/

static TMP_FILE *tmpfile_alloc(char *name)
{
    TMP_FILE *tf;

    /* Allocate memory for the structure */
    if((tf = (TMP_FILE *) malloc(sizeof(TMP_FILE))) == NULL)
        return(NULL);

    /* Allocate memory for the file name */
    if((tf->name = (char *) malloc(strlen(name) + 1)) == NULL)
        return(NULL);

    /* Copy in the file name */
    strcpy(tf->name, name);

    /* Initialize next ptr */
    tf->next = NULL;

    /* OK. */
    return(tf);
}

/* tmpfile_free:**************************************************************/
/* tmpfile_free: Free up the memory for the list.                            */
/* tmpfile_free:**************************************************************/

void tmpfile_free()
{
    TMP_FILE *tf, *free_tmpfile = first_tmpfile;

    /* Follow linked list and free each node as we go */
    while(free_tmpfile != NULL) {
        /* Delete file */
        unlink(free_tmpfile->name);
        tf = free_tmpfile->next;
        if(free_tmpfile->name) free(free_tmpfile->name);
        free(free_tmpfile);
        free_tmpfile = tf;
    }

    first_tmpfile = NULL;
}

/* tmpfile_add:***************************************************************/
/* tmpfile_add: Add package to linked list - return 1 if it is already there */
/* tmpfile_add: and 0 if it has been added. (-1) indicates an error.         */ 
/* tmpfile_add:***************************************************************/

int tmpfile_add(char *name)
{
    TMP_FILE *ptr, *tf;

    /* Create a TMP_FILE structure */
    if((tf = tmpfile_alloc(name)) == NULL)
        return(-1);

    /* First element in list */
    if(!first_tmpfile) {
        first_tmpfile = tf;
    }
    else {
        /* Find last element in list */
        for(ptr = first_tmpfile; ptr->next != NULL; ptr = ptr->next) {
            /* If it is already there then don't add it */
            if(!strcmp(tf->name, ptr->name)) {
                free(tf->name); free(name);
                return(1);
            }
        }
        ptr->next = tf;
    }

    return(0);
}

/* tmpfile_search:************************************************************/
/* tmpfile_search: Have a look in the linked list for the temporary file.    */
/* tmpfile_search:************************************************************/

int tmpfile_search(char *name)
{
    TMP_FILE *tf;

    /* Follow linked list and have a look for the filename */
    for(tf = first_tmpfile; tf != NULL; tf = tf->next) {
        if(!strcmp(name, tf->name)) {
            /* Found it's name in the list, but does it exist? */
            if(!file_exists(name))
                return(0);
            else
                return(1);
        }
    }

    /* Can't find it */
    return(0);
}

/* tmpfile_create:************************************************************/
/* tmpfile_create: Create a temporary filename from the package demo and     */
/* tmpfile_create: code.                                                     */
/* tmpfile_create:************************************************************/

char *tmpfile_create(char *package, char *code)
{
    char filename[256], *tmpname;

    /* Build up the filename in a buffer */
    sprintf(filename, "%s/%s-%s.%d", TMPDIR, package, code, getpid());

    /* Allocate memory for the new file name */
    if((tmpname = (char *) malloc(strlen(filename)+1)) == NULL)
        return(NULL);

    /* Copy in name */
    strcpy(tmpname, filename);

    /* OK. */
    return(tmpname);
}
